/// t2c_api.c
/// Implementation of T2C API specific to the testing framework used.

/////////////////////////////////////////////////////////////////////////////
// Copyright 2009-2010 
// Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//    http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 
/////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>

#include <glib.h>

#include <mist_file_utils.h>
#include <t2c_api.h>

/////////////////////////////////////////////////////////////////////////////
// Implementation: public functions
/////////////////////////////////////////////////////////////////////////////

int 
t2c_printf(const char* format, ...)
{
    assert(format != NULL);
        
    int res;
    va_list ap;
    char* buf = NULL;
    
    // Determine the size of the buffer to write to (it is res+1 on success)
    va_start(ap, format);
    res = vsnprintf(NULL, 0, format, ap);
    va_end(ap);
    
    if (res < 0)
    {
        return res;
    }
    ++res; // +1 byte for '\0' at the end
    
    buf = (char*)malloc((size_t)res);
    if (buf == NULL)
    {
        return -1;
    }
    
    // First, write the message to the string buffer
    va_start(ap, format);
    res = vsnprintf(buf, res, format, ap);
    va_end(ap);
    
    if (res >= 0)
    {
        // If everything is OK, output the contents of the buffer to the log.
        g_test_message("%s", buf);
    }
    
    free(buf);
    return res;
}

int 
t2c_print_func(const char* arg)
{
    assert(arg != NULL);
    return t2c_printf("%s", arg);
}
/////////////////////////////////////////////////////////////////////////////
