/**************************************************************************
Copyright (C) <$YEAR$> <$COPYRIGHT_HOLDER$>. All rights reserved.

<$LICENSE$>
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// This file contains tests for the following package: 
//     "<$PACKAGE_UNDER_TEST$>"
// Test group: "<$T2C_FILE_NAME$>"
// Test count: <$num_tests$>
//
// This file was created using T2C ("Template-to-Code") system:
// http://sourceforge.net/projects/template2code/
///////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <glib.h>

#include <t2c_api.h>

const char* top_suite_dir_ = TOP_SUITE_DIR;
const char* top_build_dir_ = TOP_BUILD_DIR;

const char* test_name_ = "<$T2C_FILE_NAME$>";
const char* test_path_ = TOP_BUILD_DIR "/tests/<$T2C_FILE_NAME$>/test_module";

unsigned int wait_time_ = 30;
int single_process_ = 0;  

#ifdef CHECK_EXT_REQS
    int check_ext_ = 1;
#else
    int check_ext_ = 0;
#endif

///////////////////////////////////////////////////////////////////////////
// The directory where the files containing requirements are located:
const char* rcat_dir_ = TOP_SUITE_DIR "/reqs";

// Catalogue(s) of requirements to be used (empty strings are ignored):
const char* rcat_names_[] = {
    "<$T2C_FILE_NAME$>",
    <$if concat(req_cat)$>"<$req_cat : join(",\n    ")$>",
    <$endif$>NULL
};

// Full catalogue of requirements to be used in the tests.
CReqCat* rcat_ = NULL;

///////////////////////////////////////////////////////////////////////////
// Utility functions - declarations
static void 
test_launcher(gconstpointer test_data);

static void 
startup_func();

static void 
cleanup_func();

static gboolean
t2c_test_passed(ET2CTestResult result)
{
    return  result == T2C_TR_PASSED || 
            result == T2C_TR_UNSUPPORTED ||
            result == T2C_TR_UNTESTED || 
            result == T2C_TR_FIP;
}

///////////////////////////////////////////////////////////////////////////
// User-defined globals 
<$global$>

///////////////////////////////////////////////////////////////////////////
// Startup and cleanup functions
static void 
startup_func()
{
    // initialize common settings: wait time, etc.
    wait_time_ = (unsigned int)t2c_get_number("<$WAIT_TIME$>", 
        (long)wait_time_, test_name_, t2c_print_func);
    single_process_ = (int)t2c_get_number("<$SINGLE_PROCESS$>", 
        (long)single_process_, test_name_, t2c_print_func);
    
    // load the requirements
    char* err_ = NULL;
    rcat_ = t2c_rcat_create(rcat_names_, rcat_dir_, &err_);
    if (rcat_ == NULL)
    {
        if (err_)
        {
            TRACE("Error: %s", err_);
            free(err_);
            err_ = NULL;
        }
        INIT_FAILED("<$T2C_FILE_NAME$>: failed to load catalogue of requirements");
    }

    // perform user-defined startup instructions (if any)
    <$startup$>
    // end of user-defined startup instructions    
    return;
}

static void 
cleanup_func()
{
    // perform user-defined cleanup instructions (if any)
    <$cleanup$>
    // end of user-defined cleanup instructions

    if (rcat_ != NULL)
    {
        t2c_rcat_destroy(rcat_);
    }
    return;
}


///////////////////////////////////////////////////////////////////////////
// Tests.
// The name of each test function has the following form:
//     test_<block_index>_<test_index>
// <block_index> is a 1-based index of the <BLOCK> section in the .t2c-file
//     the test was generated from;
// <test_index> is a 1-based index of the test in that <BLOCK> section.
///////////////////////////////////////////////////////////////////////////

<$test_case : join(\n)$>
    
///////////////////////////////////////////////////////////////////////////
// Specials
///////////////////////////////////////////////////////////////////////////

// The list of test functions in this file
FT2CTestFunc test_func[] = {
    <$test_func_item : join(,\n    )$>,    
    NULL
};

// The list of test names
const gchar* test_name[] = {
    <$test_name_item : join(,\n    )$>,    
    NULL
};

// Test launcher.
static void 
test_launcher(gconstpointer test_data)
{
    // 'test_data' must be a pointer to the actual test function 
    // (one of those listed in 'test_func[]' array in this file)
    g_assert(test_data != NULL);
    FT2CTestFunc test_function = (FT2CTestFunc)test_data;
    
    ET2CTestResult tres = T2C_TR_ERROR;
    char* err = NULL;
    
    int result = t2c_spawn(
        test_function,   // a test to launch
        NULL,            // no special startup function
        NULL,            // no special cleanup function
        wait_time_,      // time limit for the test (seconds); no limit if 0
        single_process_, // if 0, new process is used for each test
        &tres,           // result code of the test (T2C_TR_*)
        &err);           // error message
    
    // if the result is T2C_TR_ERROR, there is an error in the template or 
    // API library: this result should have been already handled
    g_assert(tres != T2C_TR_ERROR);
        
    if (result == 0 && err != NULL)
    {
        TRACE("Error: %s", err);
        free(err);
    }
    
    gboolean test_passed = t2c_test_passed(tres);
    if (!test_passed)
    {
        // perform cleanup here, because g_assert() will call abort() 
        // and execution will never return to main()
        cleanup_func();
    }
    
    g_assert(test_passed == TRUE);
    return;
}

///////////////////////////////////////////////////////////////////////////
int
main(int argc, char* argv[])
{
    // initilaize Glib testing system
    g_test_init(&argc, &argv, NULL);
    
    // perform startup instructions    
    startup_func();
    
    // add test cases to the suite
    int num_tests = <$num_tests$>;
    for (int i = 0; i < num_tests; ++i)
    {
        g_test_add_data_func(
            test_name[i],
            (gconstpointer)(test_func[i]),
            test_launcher
        );
    }
   
    // run the tests 
    int result = g_test_run();
    
    // perform cleanup
    cleanup_func();
    return result;
}
